SystemOrganization addCategory: #'UIManager-Support'!
SystemOrganization addCategory: #UIManager!

Exception subclass: #ProgressInitiationException
	instanceVariableNames: 'workBlock maxVal minVal aPoint progressTitle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UIManager-Support'!

!ProgressInitiationException commentStamp: '<historical>' prior: 0!
I provide a way to alter the behavior of the old-style progress notifier in String. See examples in:

ProgressInitiationException testWithout.
ProgressInitiationException testWith.
!

!ProgressInitiationException class methodsFor: 'signalling' stamp: 'ajh 1/22/2003 23:51'!
display: aString at: aPoint from: minVal to: maxVal during: workBlock

	^ self new
		display: aString at: aPoint from: minVal to: maxVal during: workBlock! !

!ProgressInitiationException class methodsFor: 'examples and tests' stamp: 'SeanDeNigris 6/21/2012 08:41'!
testInnermost

	"test the progress code WITHOUT special handling"

	^'Now here''s some Real Progress'
		displayProgressFrom: 0 to: 10
		during: [ :bar |
			1 to: 10 do: [ :x | 
				bar current: x. (Delay forMilliseconds: 500) wait.
				x = 5 ifTrue: [1/0].	"just to make life interesting"
			].
			'done'
		].

! !

!ProgressInitiationException class methodsFor: 'examples and tests' stamp: 'SeanDeNigris 6/21/2012 00:26'!
testWith

	"test progress code WITH special handling of progress notifications"

	^[ self testWithAdditionalInfo ] 
		on: ProgressInitiationException
		do: [ :ex | ex sendNotificationsTo: LoggingSystemProgressItemMorph new ].
! !

!ProgressInitiationException class methodsFor: 'examples and tests' stamp: 'RAA 5/15/2000 12:04'!
testWithAdditionalInfo

	^{'starting'. self testWithout. 'really!!'}! !

!ProgressInitiationException class methodsFor: 'examples and tests' stamp: 'RAA 5/15/2000 15:45'!
testWithout

	"test the progress code WITHOUT special handling"

	^[self testInnermost]
		on: ZeroDivide
		do: [ :ex | ex resume]

! !

!ProgressInitiationException methodsFor: 'action' stamp: 'CamilloBruni 8/30/2012 16:15'!
defaultAction

	^ UIManager default 
		displayProgress: progressTitle 
		from: minVal 
		to: maxVal 
		during: workBlock! !

!ProgressInitiationException methodsFor: 'initialize-release' stamp: 'RAA 5/15/2000 11:43'!
display: argString at: argPoint from: argMinVal to: argMaxVal during: argWorkBlock

	progressTitle := argString.
	aPoint := argPoint.
	minVal := argMinVal.
	maxVal := argMaxVal.
	workBlock := argWorkBlock.
	^self signal! !

!ProgressInitiationException methodsFor: 'accessing' stamp: 'JuanVuletich 10/28/2010 13:06'!
maxVal
	^maxVal! !

!ProgressInitiationException methodsFor: 'accessing' stamp: 'JuanVuletich 10/28/2010 13:06'!
minVal
	^minVal! !

!ProgressInitiationException methodsFor: 'accessing' stamp: 'JuanVuletich 10/28/2010 13:05'!
point
	^aPoint! !

!ProgressInitiationException methodsFor: 'accessing' stamp: 'JuanVuletich 10/28/2010 13:04'!
progressTitle
	^progressTitle! !

!ProgressInitiationException methodsFor: 'action' stamp: 'SeanDeNigris 6/21/2012 00:24'!
sendNotificationsTo: anObject

	self resume: (workBlock value: (anObject start: minVal; end: maxVal)).! !

!ProgressInitiationException methodsFor: 'accessing' stamp: 'pavel.krivanek 9/28/2006 20:31'!
workBlock 
	
	^ workBlock! !

!String methodsFor: '*UIManager-Support' stamp: 'CamilloBruni 8/30/2012 16:17'!
displayProgressAt: aPoint from: minVal to: maxVal during: workBlock 
	"Deprecated!! - Display this string as a caption over a progress bar while workBlock is evaluated.

EXAMPLE (Select next 6 lines and Do It)
'Now here''s some Real Progress'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: 10
	during: [:bar |
	1 to: 10 do: [:x | bar value: x.
			(Delay forMilliseconds: 500) wait]].

HOW IT WORKS (Try this in any other language :-)
Since your code (the last 2 lines in the above example) is in a block,
this method gets control to display its heading before, and clean up 
the screen after, its execution.
The key, though, is that the block is supplied with an argument,
named 'bar' in the example, which will update the bar image every 
it is sent the message value: x, where x is in the from:to: range.
"
	"deprecated use displayProgressFrom: minVal to: maxVal during: workBlock "

	^UIManager default 
		display: self
		from: minVal 
		to: maxVal 
		during: workBlock! !

!String methodsFor: '*UIManager-Support' stamp: 'CamilloBruni 8/30/2012 16:21'!
displayProgressFrom: minVal to: maxVal during: workBlock 
	"Display this string as a caption over a progress bar while workBlock is evaluated.

EXAMPLE (Select next 6 lines and Do It)
'Now here''s some Real Progress'
	displayProgressFrom: 0 to: 10
	during: [:bar |
	1 to: 10 do: [:x | bar value: x.
			(Delay forMilliseconds: 500) wait]].
"
	^ UIManager default 
		displayProgress: self 
		from: minVal 
		to: maxVal 
		during: workBlock! !

!AppRegistry class methodsFor: '*UIManager' stamp: 'BenjaminVanRyseghem 2/22/2012 14:41'!
chooseAppWithUI

	default := UIManager default 
		chooseFrom: (self registeredClasses collect: [:c | c name]) 
		values: self registeredClasses 
		title: ('Which ' , self appName, ' would you prefer?') translated.
	default ifNil: [default := self registeredClasses first].
	WorldState defaultWorldMenu. 
	^ default.! !

Notification subclass: #ProgressNotification
	instanceVariableNames: 'amount done extra'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UIManager-Support'!

!ProgressNotification commentStamp: '<historical>' prior: 0!
Used to signal progress without requiring a specific receiver to notify. Caller/callee convention could be to simply count the number of signals caught or to pass more substantive information with #signal:.!

!ProgressNotification class methodsFor: 'exceptioninstantiator' stamp: 'ajh 1/22/2003 23:51'!
signal: signalerText extra: extraParam
	"TFEI - Signal the occurrence of an exceptional condition with a specified textual description."

	| ex |
	ex := self new.
	ex extraParam: extraParam.
	^ex signal: signalerText! !

!ProgressNotification methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:11'!
amount
	^amount! !

!ProgressNotification methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:12'!
amount: aNumber
	amount := aNumber! !

!ProgressNotification methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:11'!
done
	^done! !

!ProgressNotification methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:12'!
done: aNumber
	done := aNumber! !

!ProgressNotification methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:12'!
extraParam
	^extra! !

!ProgressNotification methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:12'!
extraParam: anObject
	extra := anObject! !

Notification subclass: #ProvideAnswerNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UIManager-Support'!

!ClassBuilder class methodsFor: '*UIManager' stamp: 'sd 3/28/2008 11:03'!
checkClassHierarchyConsistency
	"Check the consistency of the class hierarchy. The class hierarchy is consistent if the following
	two logical equivalences hold for classes A and B:
	- B is obsolete and 'B superclass' yields A  <-->  'A obsoleteSubclasses' contains B
	- B is not obsolete and 'B superclass' yields A  <-->  'A subclasses' contains B"
	UIManager default informUserDuring: [ :bar | self checkClassHierarchyConsistency: bar ]! !

!ClassBuilder class methodsFor: '*UIManager' stamp: 'sd 3/28/2008 11:03'!
cleanupAndCheckClassHierarchy
	"Makes the class hierarchy consistent and removes obsolete classes from the SystemDictionary.
	Afterwards it checks whether the hierarchy is really consistent."
	UIManager default informUserDuring: [ :bar | self cleanupAndCheckClassHierarchy: bar ]! !

!ClassBuilder class methodsFor: '*UIManager' stamp: 'sd 3/28/2008 11:03'!
cleanupClassHierarchy
	"Makes the class hierarchy consistent and removes obsolete classes from the SystemDictionary."
	UIManager default informUserDuring: [ :bar | self cleanupClassHierarchy: bar ]! !

!ClassBuilder methodsFor: '*UIManager' stamp: 'sd 3/28/2008 11:03'!
informUserDuring: aBlock 
	self class isSilent ifTrue: [ ^ aBlock value ].
	UIManager default informUserDuring: 
		[ :bar | 
		progress := bar.
		aBlock value ].
	progress := nil! !

!UnhandledError methodsFor: '*UIManager' stamp: 'pavel.krivanek 7/4/2010 16:38'!
defaultAction

 	^ UIManager default unhandledErrorDefaultAction: self exception! !

Error subclass: #ErrorNonInteractive
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UIManager'!

!ErrorNonInteractive commentStamp: 'IgorStasenko 1/25/2011 13:29' prior: 0!
This error thrown when image runs in non-interactive mode (no user intervention possible nor expected),
but application trying to use tools which require user's intervention.
For example, in headless image application trying to query a user name, or dialog to confirm with proceeding some action.!

!ErrorNonInteractive class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 1/24/2011 10:34'!
signalForException: anError

	^ self new
		exception: anError;
		signal! !

!ErrorNonInteractive methodsFor: 'exceptiondescription' stamp: 'CamilloBruni 7/24/2012 16:22'!
defaultAction
	" log error and quit "
	
	^ UIManager default 
		quitFrom: self signalerContext 
		withMessage: self description! !

!ErrorNonInteractive methodsFor: 'accessing' stamp: 'CamilloBruni 7/24/2012 16:27'!
description
	^ String streamContents: [ :s |
		s nextPutAll: 'An attempt to use interactive tools detected, while in non-interactive mode'. 
		self messageText  ifNotEmpty: [ :message| 
			s cr; tab; nextPutAll: message ].
		self exception ifNotNilDo: [ :ex |
			s cr; tab; print: ex ]].! !

!ErrorNonInteractive methodsFor: 'accessing' stamp: 'IgorStasenko 1/24/2011 10:36'!
exception

	^ exception! !

!ErrorNonInteractive methodsFor: 'accessing' stamp: 'IgorStasenko 1/24/2011 10:36'!
exception: anError

	exception := anError! !

!Object methodsFor: '*UIManager' stamp: 'rbb 3/1/2005 09:26'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."

	"nil confirm: 'Are you hungry?'"

	^ UIManager default confirm: queryString! !

!Object methodsFor: '*UIManager' stamp: 'CamilloBruni 8/1/2012 16:09'!
inform: aString
	"Display a message for the user to read and then dismiss."

	aString isEmptyOrNil ifFalse: [UIManager default inform: aString]! !

!Object methodsFor: '*UIManager' stamp: 'pavel.krivanek 11/21/2008 16:50'!
primitiveError: aString 
	"This method is called when the error handling results in a recursion in 
	calling on error: or halt or halt:."

	UIManager default onPrimitiveError: aString.! !

Object subclass: #UIManager
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'UIManager'!

!UIManager commentStamp: 'ar 12/27/2004 08:39' prior: 0!
UIManager is a dispatcher for various UI requests.!

UIManager subclass: #CommandLineUIManager
	instanceVariableNames: 'doNotQuitOnRestart uiManager'
	classVariableNames: 'SnapshotErrorImage'
	poolDictionaries: ''
	category: 'UIManager'!

!CommandLineUIManager class methodsFor: 'instance creation' stamp: 'CamilloBruni 2/9/2012 12:02'!
replacing: aUIManager

	"Replace the current UI manager with instance of myself. 
	Keep a backup reference to old manager, and then restore it, when image will be interactive again. "
	
	^ self new replacing: aUIManager! !

!CommandLineUIManager class methodsFor: 'accessing' stamp: 'CamilloBruni 2/13/2012 21:25'!
snapshotErrorImage
	^ SnapshotErrorImage == true! !

!CommandLineUIManager class methodsFor: 'accessing' stamp: 'CamilloBruni 2/13/2012 21:25'!
snapshotErrorImage: aBoolean
	SnapshotErrorImage := aBoolean! !

!CommandLineUIManager class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 2/13/2012 21:27'!
uiSettingsOn: aBuilder
	<systemsettings>
	(aBuilder group: #nonInteractive)
		label: 'Headless mode';
		with: [
			(aBuilder setting: #snapshotErrorImage)
				label: 'Make a snapshot of new version before quit' translated;
				target: CommandLineUIManager;
				description: 'On unhandled exception, save a new version of image before quit' translated]! !

!CommandLineUIManager methodsFor: 'display' stamp: 'CamilloBruni 2/9/2012 00:21'!
checkForNewDisplaySize
	"do nothing"! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/14/2012 09:59'!
choose: questionsAnswerDict title: queryString
! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 5/26/2012 13:07'!
chooseFrom: aList lines: linesArray title: aString
	| maxPad |
	maxPad := aList size asString size.
	self stdout 
		nextPutAll: aString; nextPut: $:; cr.
	aList withIndexDo: [ :item :index |
		self stdout 
			nextPutAll: '    ['; nextPutAll: (index asString padLeftTo: maxPad); nextPutAll: '] ';
			print: item; cr].
	self stdout nextPutAll: '> '.	
	^ aList at: (self stdin upToAnyOf: String crlf do: [ :chr| ]) asInteger.! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/14/2012 09:51'!
confirm: queryString trueChoice: trueLabel falseChoice: falseLabel
	^self
		confirm: queryString
		trueChoice: trueLabel
		falseChoice: falseLabel
		cancelChoice: nil
		default: nil
		! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/14/2012 09:59'!
confirm: queryString trueChoice: trueLabel falseChoice: falseLabel cancelChoice: cancelLabel default: trueFalseNil
	| questions |
	questions := Dictionary new.
	trueLabel ifNotNil: [ questions at: 'y' put: trueLabel ].
	falseLabel ifNotNil: [ questions at: 'n' put: falseLabel ].
	cancelLabel ifNotNil: [ questions at: 'c' put: cancelLabel ].
	^ self choose: questions title: queryString
! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'CamilloBruni 2/21/2012 02:05'!
exitFailure
	[
		self class snapshotErrorImage ifTrue: [
			doNotQuitOnRestart := true.
			"make a new image version snapshot before leaving"
			Smalltalk saveAsNewVersion  ].
	] ensure: [ 
		doNotQuitOnRestart ifFalse: [ Smalltalk exitFailure ].
		doNotQuitOnRestart := false.	
	].
	
	! !

!CommandLineUIManager methodsFor: 'accessing' stamp: 'CamilloBruni 2/13/2012 21:23'!
headlessManager
	self class == CommandLineUIManager 
		ifFalse: [ ^ self ].
		
	^ CommandLineUIManager replacing: uiManager! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/13/2012 18:29'!
inform: aString

	| logBlock |

	"Just log notifications"
	(ProvideAnswerNotification signal: aString) ifNotNil: [:answer | ^true].
	
	logBlock := [:logger |
		logger cr;
			nextPutAll: (String new: 79 withAll: $= ); cr;
			nextPutAll: 'Notice: ';
			nextPutAll: aString; cr;
			nextPutAll: (String new: 79 withAll: $= ); cr].

	Smalltalk logDuring: logBlock.
	self logYellowDuring: logBlock.
		
	Transcript show: aString; cr.! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 8/30/2012 16:19'!
informUserDuring: aBlock
	self
		displayProgress: ''
		from: 1 to: 100
		during: aBlock! !

!CommandLineUIManager methodsFor: 'initialize-release' stamp: 'CamilloBruni 2/9/2012 00:22'!
initialize
	doNotQuitOnRestart := false.! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'IgorStasenko 3/12/2012 17:49'!
logColored: anAnsiiColorCode during: aBlock
	
	Smalltalk logStdErrorDuring: [:stderr |
		stderr
			nextPut: Character escape; 
			nextPut: $[; nextPutAll: anAnsiiColorCode; nextPut: $m.
		
		aBlock value: stderr.
		
		stderr nextPut: Character escape; nextPutAll: '[0m'
	]! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'IgorStasenko 3/12/2012 17:49'!
logDuring: aBlock
	
	Smalltalk logStdErrorDuring: [ :stderr | aBlock value: stderr ]

! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'CamilloBruni 2/9/2012 12:55'!
logGreenDuring: aBlock
	
	^ self logColored: '32' during: aBlock! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'CamilloBruni 2/9/2012 12:53'!
logRedDuring: aBlock
	
	^ self logColored: '31' during: aBlock! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'CamilloBruni 2/9/2012 12:55'!
logYellowDuring: aBlock
	
	^ self logColored: '33' during: aBlock! !

!CommandLineUIManager methodsFor: 'display' stamp: 'CamilloBruni 2/9/2012 00:21'!
newDisplayDepthNoRestore: pixelSize	
	"do nothing"
	
! !

!CommandLineUIManager methodsFor: 'non-interactive' stamp: 'CamilloBruni 2/13/2012 19:14'!
nonInteractiveManager
	" Answer an instance of non-interactive manager, which will be used when image runs headless.
	  We put it here, so subclasses can override it. "
	^ NonInteractiveUIManager replacing: uiManager! !

!CommandLineUIManager methodsFor: 'events' stamp: 'CamilloBruni 2/13/2012 19:11'!
onPrimitiveError: aString	

	" log error and quit "
	^ self quitFrom: thisContext sender withMessage: aString! !

!CommandLineUIManager methodsFor: 'events' stamp: 'FernandoOlivero 3/16/2012 16:24'!
onSnapshot: resuming
	"The resuming argument is true when image boots from disk,
	and false, if user just did an image snapshot."
	
	resuming ifTrue: [
		Smalltalk isInteractive ifFalse: [
			^ self nonInteractiveManager onSnapshot: resuming ].
		Smalltalk isHeadless ifFalse: [
			uiManager beDefault.   "restore old, or nil, so it will be set in #default "
			UIManager default onSnapshot: resuming]. 
		^ self].
	
	
	" this flag set to true only if we are saving a snapshot before quitting "
	doNotQuitOnRestart ifTrue: [
		Smalltalk snapshot: false andQuit: true].! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'SeanDeNigris 6/11/2012 11:28'!
progressInitiationExceptionDefaultAction: anException

	| result |

	result := anException workBlock value: DummySystemProgressItem new.

	anException resume: result! !

!CommandLineUIManager methodsFor: 'utils' stamp: 'CamilloBruni 2/21/2012 02:05'!
quitFrom: aContext withMessage: aString

	" log error and quit "
	[ Smalltalk 
		logError: aString
		inContext: aContext.
		
	" Print stacks of all current processes "

	Smalltalk logDuring: [:logger | 
		logger nextPutAll: 'Processes and their stacks: ';cr.
		
		Process allInstances do: [:each | | ctx |
			logger nextPutAll: 'Process: '; print: each; cr; nextPutAll: '  stack:'; cr; cr.
		
			ctx := each isActiveProcess ifTrue: [ thisContext sender ] ifFalse: [ each suspendedContext ].
			ctx ifNotNil: [
				(ctx stackOfSize: 20) do: [:s | logger print: s; cr ]].
			logger nextPutAll: '------------------------------'; cr; cr.	
		]].
	] ensure: [ self exitFailure ]! !

!CommandLineUIManager methodsFor: 'nil' stamp: 'CamilloBruni 2/9/2012 12:02'!
replacing: aUIManager

	" save the aUIManager to restore it later, when we become interactive again "
	
	uiManager := aUIManager.
	
	self beDefault.! !

!CommandLineUIManager methodsFor: 'display' stamp: 'CamilloBruni 2/9/2012 00:21'!
restoreDisplay	
	"do nothing"! !

!CommandLineUIManager methodsFor: 'display' stamp: 'CamilloBruni 2/9/2012 00:21'!
restoreDisplayAfter: aBlock	
	"do nothing"! !

!CommandLineUIManager methodsFor: 'accessing' stamp: 'CamilloBruni 2/13/2012 17:06'!
stderr
	
	"install the line end conversion and initialize the converter"
	 FileStream stderr wantsLineEndConversion: true; converter.
	
	^ FileStream stderr! !

!CommandLineUIManager methodsFor: 'accessing' stamp: 'CamilloBruni 2/13/2012 18:40'!
stdin
	
	^ FileStream stdin! !

!CommandLineUIManager methodsFor: 'accessing' stamp: 'CamilloBruni 2/13/2012 18:37'!
stdout
	
	"install the line end conversion and initialize the converter"
	 FileStream stdout wantsLineEndConversion: true; converter.
	
	^ FileStream stdout! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 5/26/2012 13:08'!
syntaxErrorNotificationDefaultAction: aSyntaxErrorNotification
	"log the syntax notificaiton and print a nicely formatted and colored syntax error on stderr"
	| contents position errorMessage lineNumber maxLineNumberSize errorLine |
	
	"log the error"
	Smalltalk logDuring: [ :logger |
		logger print: aSyntaxErrorNotification; cr.
		aSyntaxErrorNotification signalerContext errorReportOn: logger ].
	
	"format the error"
	position := aSyntaxErrorNotification location.
	contents := aSyntaxErrorNotification errorCode.
	errorLine := contents lineNumberCorrespondingToIndex: position.
		
	"first gather the error title to be able to underline it properly"
	errorMessage := String streamContents: [ :s|
		s nextPutAll: 'Syntax Error on line '; 
			print: errorLine; nextPutAll: ': '; 
			print: aSyntaxErrorNotification errorMessage].
	
	self logRedDuring: [ :s|
		s nextPutAll: errorMessage; cr;
			nextPutAll: ('' padLeftTo: errorMessage size with: $=); cr].
	
	"print each source line and mark the found syntax error"
	maxLineNumberSize := (contents lines size) asString size.
	lineNumber := 0.
	contents lineIndicesDo: [:start :endWithoutDelimiters :end |
		lineNumber := lineNumber + 1.
		self logColored:  (lineNumber == errorLine ifTrue:['31'] ifFalse:['33']) during: [ :s|
			"0 pad the line numbers to the same size"
			s nextPutAll: ( lineNumber asString padLeftTo: maxLineNumberSize with: $0); nextPutAll: ': ' ].
		
		self stderr nextPutAll: (contents copyFrom: start to: endWithoutDelimiters); cr.
		"print the marker under the error line"
		(lineNumber == errorLine) ifTrue: [
			self logRedDuring: [ :s|
				s nextPutAll:( '_^_' padLeftTo: position - start + maxLineNumberSize + 4); cr]]].
			
	"in noninteractive mode simply quit"
	^ self exitFailure! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/13/2012 19:12'!
uiProcess
	" receiver don't have a ui process, associated with it,
	client should check explicitly if #uiProcess answers nil or not"
	^ nil! !

!CommandLineUIManager methodsFor: 'default actions' stamp: 'CamilloBruni 2/13/2012 19:12'!
unhandledErrorDefaultAction: anException

	self quitFrom: anException signalerContext  withMessage: anException description.
	
	UIManager default == self ifFalse: [
		^ UIManager default unhandledErrorDefaultAction: anException
		]! !

!CommandLineUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/13/2012 19:09'!
warningDefaultAction: aWarning
	| logBlock |
	"Pass all warnings, but log them"
	
	logBlock := [:logger |
		logger 
			cr;
			nextPutAll: '*** Warning: ';
			nextPutAll: aWarning description;
			cr ].
	
	Smalltalk logDuring: logBlock.
	self logYellowDuring: logBlock.
	
	aWarning resume.

	! !

CommandLineUIManager subclass: #NonInteractiveUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UIManager'!

!NonInteractiveUIManager commentStamp: 'IgorStasenko 1/24/2011 15:36' prior: 0!
This is a non-interactive UI manager, i.e. a UI manager which doesn't provides any kind of interactivity with users.

For most of requests, it throws an ErrorNonInteractive exception, which can be handled by various tools to do things differently when UI is not avaliable. For example:

response := [ UIManager default request: 'what is your name?' ] on: ErrorNonInteractive do: [:ex | ex resume: 'Mr. John Smith' ].

You can replace the default UI Manager with my instance in cases, when you need to guarantee that your task(s) will run in fully automated mode. This is useful for things like:
  - when image runs as a persistent application on a server
  - image runs headless from command-line with some batch scripts/commands

!

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:21'!
chooseDirectory: label from: dir
	"Let the user choose a directory"
	^ self nonInteractiveWarning: (String streamContents: [ :s|
			s 
				nextPutAll: 'Interactive Directory Choice Request: ';
				nextPutAll: '[ '; nextPutAll: label asString; nextPutAll: ' ] from '; print: dir
		])! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:20'!
chooseFileMatching: patterns label: labelString
	"Let the user choose a file matching the given patterns"
	^ self chooseFullFileNameMatching: patterns label: labelString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:18'!
chooseFrom: aList lines: linesArray title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	^ self chooseOrRequestFrom: aList values: aList lines: linesArray title: aString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:18'!
chooseFrom: labelList values: valueList lines: linesArray title: aString
	"Choose an item from the given list. Answer the selected item."
	^ self chooseOrRequestFrom: labelList values: valueList lines: linesArray title: aString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:19'!
chooseFullFileNameMatching: patterns label: labelString
	"Let the user choose a file matching the given patterns"
	^ self nonInteractiveWarning: (String streamContents: [ :s|
			s 
				nextPutAll: 'Interactive File Choice Request: ';
				nextPutAll: '[ '; nextPutAll: labelString asString; nextPutAll: ' ] ';
				print: patterns.
		])! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:31'!
chooseOrRequestFrom: labelList values: valueList lines: linesArray title: aString
	 ^ self nonInteractiveWarning: (String streamContents: [ :s|
			s nextPutAll: 'Choice Request: '; nextPutAll: aString; cr.
			labelList with: valueList do: [ :label :value| 
				s tab; nextPutAll: label asString; nextPutAll: ':'; tab; print: value; cr ]])! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:12'!
confirm: queryString
	(ProvideAnswerNotification signal: queryString) ifNotNil: [:answer |
		^ answer].
	
	^  self nonInteractiveWarning: 'Confirming: ', queryString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:12'!
confirm: queryString label: titleString
	^  self nonInteractiveRequest: queryString title: titleString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:12'!
confirm: aString orCancel: cancelBlock
	^  self nonInteractiveRequest: aString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:12'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice 
	^  self nonInteractiveRequest: queryString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:12'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice cancelChoice: cancelChoice default: aSymbol
	^  self nonInteractiveRequest: queryString! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:08'!
edit: aText label: labelString accept: anAction
	"Open an editor on the given string/text"
	^self nonInteractiveWarning: (String streamContents: [ :s |
		s  nextPutAll: 'Editing Text: '.
		labelString ifNotNil: [ s nextPutAll: '[ '; nextPutAll: labelString; nextPutAll: ' ] '].
		s cr; nextPutAll: aText])! !

!NonInteractiveUIManager methodsFor: 'default actions' stamp: 'IgorStasenko 1/24/2011 10:33'!
fileDoesNotExistsDefaultAction: anException

	^ self nonInteractive: anException ! !

!NonInteractiveUIManager methodsFor: 'default actions' stamp: 'IgorStasenko 1/24/2011 14:37'!
fileExistsDefaultAction: anException

	^ self nonInteractive: anException ! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:06'!
fontFromUser: priorFont	
	^self nonInteractiveWarning: 'Font Choose Request'! !

!NonInteractiveUIManager methodsFor: 'settings' stamp: 'MarcusDenker 1/26/2011 14:12'!
interactiveParserFor: requestor	
	^ (requestor respondsTo: #interactive) and: [ requestor interactive ]! !

!NonInteractiveUIManager methodsFor: 'default actions' stamp: 'CamilloBruni 2/9/2012 00:25'!
lowSpaceWatcherDefaultAction: preemptedProcess
	| ctx |
	"yeah.. space is low.. quit!!!! :)"
	
	ctx := preemptedProcess ifNotNil: [  preemptedProcess suspendedContext ].
	ctx ifNil: [ ctx := thisContext sender ]. 
	self quitFrom: ctx withMessage: 'Low space signal received.'.
	
	self halt. "what else we can do? "! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:10'!
multiLineRequest: queryString initialAnswer: defaultAnswer answerHeight: answerHeight
	^  self nonInteractiveRequest: queryString! !

!NonInteractiveUIManager methodsFor: 'errors' stamp: 'IgorStasenko 1/21/2011 16:25'!
nonInteractive
	^ ErrorNonInteractive signal! !

!NonInteractiveUIManager methodsFor: 'errors' stamp: 'IgorStasenko 1/24/2011 10:34'!
nonInteractive: anException
	^ ErrorNonInteractive signalForException: anException! !

!NonInteractiveUIManager methodsFor: 'non-interactive' stamp: 'CamilloBruni 2/13/2012 19:02'!
nonInteractiveManager
	" Answer an instance of non-interactive manager, which will be used when image runs headless.
	  We put it here, so subclasses can override it. 
	
	We already non-interactive. Just answer self
	"
	^ self! !

!NonInteractiveUIManager methodsFor: 'errors' stamp: 'CamilloBruni 7/24/2012 16:11'!
nonInteractiveRequest: aStringOrText
	^ self nonInteractiveRequest: aStringOrText title: nil! !

!NonInteractiveUIManager methodsFor: 'errors' stamp: 'CamilloBruni 7/24/2012 16:09'!
nonInteractiveRequest: aStringOrText title: aTitle
	 ^ self nonInteractiveWarning: (String streamContents: [ :s|
			s nextPutAll: 'Interactive Request: '.
			aTitle ifNotNil: [ s nextPutAll: '[ '; nextPutAll: aTitle asString; nextPutAll: ' ] ' ].
			s nextPutAll: aStringOrText asString.
		])! !

!NonInteractiveUIManager methodsFor: 'errors' stamp: 'CamilloBruni 7/24/2012 15:57'!
nonInteractiveWarning: aWarnMessage
	^ ErrorNonInteractive signal: aWarnMessage! !

!NonInteractiveUIManager methodsFor: 'events' stamp: 'CamilloBruni 7/24/2012 16:05'!
onDebug: process context: context title: title full: bool

	^ self nonInteractiveWarning: 'Opening Debugger'! !

!NonInteractiveUIManager methodsFor: 'events' stamp: 'FernandoOlivero 3/16/2012 16:24'!
onSnapshot: resuming
	"The resuming argument is true when image boots from disk,
	and false, if user just did an image snapshot."
	
	resuming ifTrue: [
		Smalltalk isHeadless ifFalse: [
			"restore old, or nil, so it will be set in #default "
			uiManager beDefault.  
			UIManager default onSnapshot: resuming.
			^ self ].
		Smalltalk isInteractive ifTrue: [
			"use a headless but interactive manager"
			^ self headlessManager onSnapshot: resuming ]].
	
	
	" this flag set to true only if we are saving a snapshot before quitting "
	doNotQuitOnRestart ifTrue: [
		Smalltalk snapshot: false andQuit: true].! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:02'!
request: queryString initialAnswer: defaultAnswer
	^ self request: queryString initialAnswer: defaultAnswer title: nil entryCompletion: nil. ! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:02'!
request: queryString initialAnswer: defaultAnswer centerAt: aPoint
	"Create an instance of me whose question is queryString with the given initial answer. Answer the string the user accepts. Answer the empty string if the user cancels."
	
	^ self request: queryString initialAnswer: defaultAnswer title: nil entryCompletion: nil. ! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:02'!
request: queryString initialAnswer: defaultAnswer entryCompletion: anEntryCompletion
	^ self request: queryString initialAnswer: defaultAnswer title: nil entryCompletion: anEntryCompletion. ! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:02'!
request: aStringOrText initialAnswer: defaultAnswer title: aTitle
	^ self request: aStringOrText initialAnswer: defaultAnswer title: aTitle entryCompletion: nil. ! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 16:10'!
request: aStringOrText initialAnswer: defaultAnswer title: aTitle entryCompletion: anEntryCompletion
	(ProvideAnswerNotification signal: aStringOrText)
		ifNotNil: [:answer | 
			^ answer = #default ifTrue: [ defaultAnswer ] ifFalse: [ answer ]].
		
	^ self nonInteractiveRequest: aStringOrText title: aTitle! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 7/24/2012 15:56'!
requestPassword: queryString
	^self nonInteractiveWarning: 'Requesting Password'.! !

!NonInteractiveUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 2/21/2012 02:05'!
syntaxErrorNotificationDefaultAction: aSyntaxErrorNotification
	
	"display and log the syntax error"
	super syntaxErrorNotificationDefaultAction: aSyntaxErrorNotification.
	
	"in noninteractive mode simply quit"
	^ self exitFailure! !

NonInteractiveUIManager subclass: #StartupUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UIManager'!

!StartupUIManager methodsFor: 'ui requests' stamp: 'StephaneDucasse 6/7/2011 15:52'!
inform: aString

	(ProvideAnswerNotification signal: aString) ifNotNil: [:answer | ^true].
	Smalltalk addDeferredStartupAction: [ UIManager default inform: aString ].! !

UIManager subclass: #DummyUIManager
	instanceVariableNames: ''
	classVariableNames: 'ProgressBarEnabled'
	poolDictionaries: ''
	category: 'UIManager'!

!DummyUIManager commentStamp: 'LaurentLaffont 2/23/2011 20:16' prior: 0!
I'm an alternative UIManager used to run an the image without GUI.  I redefine methods which requires user input as these requests are irrelevant in a headless environment. !

!DummyUIManager methodsFor: 'display' stamp: 'pavel.krivanek 5/31/2007 08:20'!
checkForNewDisplaySize

	Display extent = DisplayScreen actualScreenSize ifTrue: [^ self].
	DisplayScreen startUp.
! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:20'!
chooseDirectory: label from: dir

	^ nil! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:20'!
chooseFrom: aList lines: linesArray title: aString

	^ aList first! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:20'!
chooseFrom: labelList values: valueList lines: linesArray title: aString

	^ valueList first! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'MarcusDenker 4/29/2011 00:33'!
confirm: queryString

	(ProvideAnswerNotification signal: queryString) ifNotNil: [:answer |
		^answer].

	self error: 'No user response possible'! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'MarcusDenker 4/29/2011 00:33'!
confirm: aString orCancel: cancelBlock

	(ProvideAnswerNotification signal: aString) ifNotNil: [:answer |
		^answer == #cancel ifTrue: [cancelBlock value] ifFalse: [answer]].

	self error: 'No user response possible'! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:20'!
edit: aText label: labelString accept: anAction

	^ nil! !

!DummyUIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 5/31/2007 08:20'!
fileDoesNotExistsDefaultAction: anException
! !

!DummyUIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 5/31/2007 08:20'!
fileExistsDefaultAction: anException
! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:23'!
fontFromUser: priorFont

	self error: 'No user response possible'! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'CamilloBruni 9/22/2012 20:17'!
inform: aString
	"Nothing to be done here"! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'SeanDeNigris 6/20/2012 23:32'!
informUserDuring: aBlock

	aBlock value: DummySystemProgressItem new.! !

!DummyUIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 5/31/2007 08:20'!
lowSpaceWatcherDefaultAction

	Transcript show: '*** LOW SPACE ***'; cr.
! !

!DummyUIManager methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 9/7/2011 21:21'!
multiLineRequest: queryString initialAnswer: defaultAnswer answerHeight: answerHeight! !

!DummyUIManager methodsFor: 'display' stamp: 'pavel.krivanek 5/31/2007 08:20'!
newDisplayDepthNoRestore: pixelSize

	"Change depths.  Check if there is enough space!!  , di"
	| area need |
	pixelSize = Display depth ifTrue: [^ self  "no change"].
	pixelSize abs < Display depth ifFalse:
		["Make sure there is enough space"
		area := Display boundingBox area. "pixels"

		need := (area * (pixelSize abs - Display depth) // 8)  "new bytes needed"
				+ Smalltalk lowSpaceThreshold.
		(Smalltalk garbageCollectMost <= need
			and: [Smalltalk garbageCollect <= need])
			ifTrue: [self error: 'Insufficient free space']].
	Display setExtent: Display extent depth: pixelSize.

	DisplayScreen startUp! !

!DummyUIManager methodsFor: 'accessing' stamp: 'PavelKrivanek 11/1/2010 20:18'!
progressBarEnabled

  ^ ProgressBarEnabled ifNil: [ ProgressBarEnabled := true ].! !

!DummyUIManager methodsFor: 'accessing' stamp: 'PavelKrivanek 11/1/2010 20:15'!
progressBarEnabled: aBoolean

  ProgressBarEnabled := aBoolean! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'SeanDeNigris 6/22/2012 09:35'!
progressInitiationExceptionDefaultAction: anException

	| result |

	result := anException workBlock value: DummySystemProgressItem new.

	anException resume: result.! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:23'!
request: queryString initialAnswer: defaultAnswer 

	^ defaultAnswer! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 11/26/2010 10:18'!
request: queryString initialAnswer: defaultAnswer entryCompletion: anEntryCompletion

	^ defaultAnswer! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 11/26/2010 10:20'!
request: aStringOrText initialAnswer: defaultAnswer title: aTitle entryCompletion: anEntryCompletion

	^ defaultAnswer! !

!DummyUIManager methodsFor: 'ui requests' stamp: 'pavel.krivanek 5/31/2007 08:20'!
requestPassword: queryString
	
	^ self request: queryString initialAnswer: ''! !

!DummyUIManager methodsFor: 'display' stamp: 'pavel.krivanek 5/31/2007 08:20'!
restoreDisplay! !

!DummyUIManager methodsFor: 'display' stamp: 'pavel.krivanek 5/31/2007 08:20'!
restoreDisplayAfter: aBlock

	aBlock value.
	Sensor waitButton.! !

!DummyUIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 5/31/2007 08:20'!
syntaxErrorNotificationDefaultAction: anException

	Transcript show: '*** SYNTAX ERROR ***'; cr.
	Transcript show: anException; cr.
	Transcript show: (thisContext stack first: (20 min: thisContext stack size)); cr.
	anException return.! !

!DummyUIManager methodsFor: 'default actions' stamp: 'pavel.krivanek5/31/2007 08:20'!
unhandledErrorDefaultAction: anException

	Transcript show: '*** EXCEPTION ***'; cr.
	Transcript show: anException; cr.
	Transcript show: (thisContext stack first: (20 min: thisContext stack size)); cr.
	anException isResumable ifTrue: [ anException resume ]
! !

!DummyUIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 5/11/2008 17:44'!
warningDefaultAction: anException

	^ self unhandledErrorDefaultAction: anException! !

!UIManager class methodsFor: 'utils' stamp: 'nice 1/5/2010 15:59'!
createPageTestWorkspace
	"Used to generate a workspace window for testing page up and page down stuff."
	"self createPageTestWorkspace"

	| numberOfLines maxStringLength minLineCounterSize lineCounterSize offsetSize stream headerConstant prevStart prevStrLen prevLineNumber |
	numberOfLines := 400.
	maxStringLength := 22.
	minLineCounterSize := 3.
	lineCounterSize := (numberOfLines log asInteger + 1) max: minLineCounterSize.
	offsetSize := 5.
	stream := ReadWriteStream on: ''.
	headerConstant := lineCounterSize + 1 + offsetSize + 1.
	prevStart := headerConstant negated.
	prevStrLen := 0.
	prevLineNumber := 0.
	numberOfLines timesRepeat: [ | pad lineNumber stringLen start charIndex log |
		stringLen := maxStringLength atRandom max: 1.
		lineNumber := prevLineNumber + 1.
		start := prevStart + prevStrLen + headerConstant + 1.
		prevStart := start.
		prevStrLen := stringLen.
		prevLineNumber := lineNumber.
		log := lineNumber log asInteger.
		pad := lineCounterSize - log - 1.
		pad timesRepeat: [stream nextPutAll: '0'].
		stream nextPutAll: lineNumber printString.
		stream space.
		log := start log asInteger.
		pad := offsetSize - log - 1.
		pad timesRepeat: [stream nextPutAll: '0'].
		stream nextPutAll: start printString.
		stream space.
		charIndex := 'a' first asInteger.
		stringLen timesRepeat: [ | char |
			char := Character value: charIndex.
			charIndex := charIndex + 1.
			stream nextPut: char].
		lineNumber = numberOfLines ifFalse: [stream cr]
		].
	self default edit: stream contents label: 'Test Data'.
! !

!UIManager class methodsFor: 'initialization' stamp: 'IgorStasenko 1/24/2011 15:47'!
default
	| mgrClass |
	^Default ifNil:[
		"Note: The way the following is phrased ensures that you can always make 'more specific' managers merely by subclassing a tool builder and implementing a more specific way of reacting to #isActiveManager. For example, a BobsUIManager can subclass MorphicUIManager and will be considered before the parent (generic MorphicUIManager)."
		mgrClass := self allSubclasses 
			detect:[:any| any isActiveManager and:[
				any subclasses noneSatisfy:[:sub| sub isActiveManager]]] ifNone:[nil].
		mgrClass ifNotNil:[Default := mgrClass new]
	].! !

!UIManager class methodsFor: 'initialization' stamp: 'SeanDeNigris 8/30/2012 11:25'!
default: aUIManager

	Default deactivate.
	Default := aUIManager.
	Default activate.! !

!UIManager class methodsFor: 'initialization' stamp: 'ar 2/11/2005 15:41'!
isActiveManager
	"Answer whether I should act as the active ui manager"
	^false! !

!UIManager class methodsFor: 'utils' stamp: 'DamienPollet 10/26/2011 17:23'!
nonInteractiveDuring: aBlock
	| currentManager |
	currentManager := self default.
	currentManager nonInteractiveManager.
	
	aBlock ensure: [ currentManager beDefault ]! !

!UIManager methodsFor: 'private' stamp: 'SeanDeNigris 8/30/2012 11:26'!
activate! !

!UIManager methodsFor: 'private' stamp: 'SeanDeNigris 8/30/2012 11:21'!
beDefault

	self class default: self.! !

!UIManager methodsFor: 'image startup' stamp: 'SeanDeNigris 8/30/2012 11:23'!
boot: bootingFromDisk during: aBlock

	bootingFromDisk ifTrue: [ 
		"use non-interactive manager while processing startup list"
		self class default: (StartupUIManager replacing: self) ].

	aBlock ensure: [ self class default: self ].

	self onSnapshot: bootingFromDisk! !

!UIManager methodsFor: 'display' stamp: 'Pavel.Krivanek 10/28/2008 11:05'!
checkForNewDisplaySize	
	
	self subclassResponsibility ! !

!UIManager methodsFor: 'ui requests' stamp: 'StephaneDucasse 6/22/2012 18:49'!
chooseDirectory
	"Let the user choose a directory"
	^self chooseDirectoryFrom: FileSystem workingDirectory! !

!UIManager methodsFor: 'ui requests' stamp: 'StephaneDucasse 6/22/2012 18:49'!
chooseDirectory: label
	"Let the user choose a directory"
	^self chooseDirectory: label from: FileSystem workingDirectory! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/16/2005 19:36'!
chooseDirectory: label from: dir
	"Let the user choose a directory"
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/16/2005 19:36'!
chooseDirectoryFrom: dir
	"Let the user choose a directory"
	^self chooseDirectory: nil from: dir! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/17/2005 00:26'!
chooseFileMatching: patterns
	"Let the user choose a file matching the given patterns"
	^self chooseFileMatching: patterns label: nil! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/17/2005 00:26'!
chooseFileMatching: patterns label: labelString
	"Let the user choose a file matching the given patterns"
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 10:44'!
chooseFrom: aList
	"Choose an item from the given list. Answer the index of the selected item."
	^self chooseFrom: aList lines: #()! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 10:45'!
chooseFrom: aList lines: linesArray
	"Choose an item from the given list. Answer the index of the selected item."
	^self chooseFrom: aList lines: linesArray title: ''! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:45'!
chooseFrom: aList lines: linesArray message: messageString
	"Choose an item from the given list. Answer the index of the selected item."
	
	^self chooseFrom: aList lines: linesArray message: messageString title: ''! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:47'!
chooseFrom: aList lines: linesArray message: messageString title: aString
	"Choose an item from the given list. Answer the selected item."
	
	^self
		chooseFrom: aList
		lines: linesArray
		title: (aString 
				ifEmpty: [messageString]
				ifNotEmpty: [aString, String cr, messageString])! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 09:37'!
chooseFrom: aList lines: linesArray title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:45'!
chooseFrom: aList message: messageString
	"Choose an item from the given list. Answer the index of the selected item."
	
	^self chooseFrom: aList lines: #() message: messageString! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:44'!
chooseFrom: aList message: messageString title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	
	^self chooseFrom: aList lines: #() message: messageString title: aString! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 10:44'!
chooseFrom: aList title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	^self chooseFrom: aList lines: #() title: aString! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/15/2005 23:42'!
chooseFrom: labelList values: valueList
	"Choose an item from the given list. Answer the selected item."
	^self chooseFrom: labelList values: valueList lines: #()! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/15/2005 23:43'!
chooseFrom: labelList values: valueList lines: linesArray
	"Choose an item from the given list. Answer the selected item."
	^self chooseFrom: labelList values: valueList lines: linesArray title: ''! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:50'!
chooseFrom: aList values: valueList lines: linesArray message: messageString
	"Choose an item from the given list. Answer the index of the selected item."
	
	^self chooseFrom: aList values: valueList lines: linesArray message: messageString title: ''! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:52'!
chooseFrom: labelList values: valueList lines: linesArray message: messageString title: aString
	"Choose an item from the given list. Answer the selected item."
	
	^self
		chooseFrom: labelList
		values: valueList 
		lines: linesArray
		title: (aString 
				ifEmpty: [messageString]
				ifNotEmpty: [aString, String cr, messageString])! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/15/2005 23:43'!
chooseFrom: labelList values: valueList lines: linesArray title: aString
	"Choose an item from the given list. Answer the selected item."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:49'!
chooseFrom: aList values: valueList message: messageString
	"Choose an item from the given list. Answer the index of the selected item."
	
	^self chooseFrom: aList values: valueList lines: #() message: messageString! !

!UIManager methodsFor: 'ui requests' stamp: 'gvc 2/6/2009 12:50'!
chooseFrom: aList values: valueList message: messageString title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	
	^self chooseFrom: aList values: valueList lines: #() message: messageString title: aString! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/15/2005 23:43'!
chooseFrom: labelList values: valueList title: aString
	"Choose an item from the given list. Answer the selected item."
	^self chooseFrom: labelList values: valueList lines: #() title: aString! !

!UIManager methodsFor: 'ui requests' stamp: 'CarloTeixeira 5/23/2011 15:40'!
chooseFullFileNameMatching: patterns
	"Let the user choose a file matching the given patterns"
	^self chooseFullFileNameMatching: patterns label: nil! !

!UIManager methodsFor: 'ui requests' stamp: ''!
chooseFullFileNameMatching: patterns label: labelString
	"Let the user choose a file matching the given patterns"
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'EstebanLorenzano 11/21/2009 17:01'!
chooseOrRequestFrom: aList title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	^self chooseOrRequestFrom: aList lines: #() title: aString! !

!UIManager methodsFor: 'ui requests' stamp: 'EstebanLorenzano 11/21/2009 10:48'!
chooseOrRequestFrom: labelList values: valueList lines: linesArray title: aString
	self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:39'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'jrd 5/12/2009 18:02'!
confirm: queryString label: titleString
	
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 09:49'!
confirm: aString orCancel: cancelBlock
	"Put up a yes/no/cancel menu with caption aString. Answer true if  
	the response is yes, false if no. If cancel is chosen, evaluate  
	cancelBlock. This is a modal question--the user must respond yes or no."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'dc 1/10/2008 08:09'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice 
	"Put up a yes/no menu with caption queryString. The actual wording for the two choices will be as provided in the trueChoice and falseChoice parameters. Answer true if the response is the true-choice, false if it's the false-choice.
	This is a modal question -- the user must respond one way or the other."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'GaryChambers 8/23/2010 12:11'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice cancelChoice: cancelChoice default: aSymbol
	"Put up a yes/no/cancel menu with caption queryString. The actual wording for the choices will be as provided in the trueChoice, falseChoice and cancelChoice parameters.
	Default should be one of #yes, #no or #cancel to set the default button.
	Answer true if the response is the true-choice, false if it's the false-choice, nil if the cancelChoice.
	This is a modal question -- the user must respond."
	
	^self subclassResponsibility! !

!UIManager methodsFor: 'private' stamp: 'SeanDeNigris 8/30/2012 11:20'!
deactivate! !

!UIManager methodsFor: 'deprecated' stamp: 'CamilloBruni 8/30/2012 16:18'!
displayProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock
	"Display titleString as a caption over a progress bar while workBlock is evaluated."
	^ self 
		displayProgress: titleString 
		from: minVal 
		to: maxVal 
		during: workBlock! !

!UIManager methodsFor: 'display' stamp: 'CamilloBruni 8/30/2012 16:20'!
displayProgress: titleString from: minVal to: maxVal during: workBlock
	"SystemProgressMorph show: titleString from: minVal to:  during: "

	^ workBlock asJob
			title: titleString;
			min: minVal;
			max: maxVal;
			run.! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/16/2005 18:56'!
edit: aText
	"Open an editor on the given string/text"
	^self edit: aText label: nil! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/16/2005 18:56'!
edit: aText label: labelString
	"Open an editor on the given string/text"
	^self edit: aText label: labelString accept: nil! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 7/16/2005 18:56'!
edit: aText label: labelString accept: anAction
	"Open an editor on the given string/text"
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 4/15/2010 10:39'!
enterOrRequestFrom: aList title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	^self enterOrRequestFrom: aList lines: #() title: aString! !

!UIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 6/17/2010 09:23'!
fileDoesNotExistsDefaultAction: anException

	^ self subclassResponsibility ! !

!UIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 6/17/2010 09:19'!
fileExistsDefaultAction: anException

	^ self subclassResponsibility ! !

!UIManager methodsFor: 'ui requests' stamp: 'Pavel.Krivanek 10/28/2008 10:45'!
fontFromUser: priorFont	
	
	self subclassResponsibility! !

!UIManager methodsFor: 'accessing' stamp: 'CamilloBruni 2/9/2012 13:06'!
headlessManager
	" Answer an instance of headless manager, which will be used when image runs headless and non-interactive.
	  We put it here, so subclasses can override it. "
	^ CommandLineUIManager replacing: self! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:46'!
inform: aString
	"Display a message for the user to read and then dismiss"
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'SeanDeNigris 6/20/2012 23:35'!
informUser: aString during: aBlock
	"Display a message above (or below if insufficient room) the cursor 
	during execution of the given block.
		UIManager default informUser: 'Just a sec!!' during: [(Delay forSeconds: 1) wait].
	"
	^self informUserDuring: [:bar | bar label: aString. aBlock value].! !

!UIManager methodsFor: 'ui requests' stamp: 'SeanDeNigris 6/20/2012 23:29'!
informUserDuring: aBlock
	"Display a message above (or below if insufficient room) the cursor 
	during execution of the given block.
		UIManager default informUserDuring:[:bar|
			#(one two three) do:[:info|
				bar label: info.
				(Delay forSeconds: 1) wait]]"
	^self subclassResponsibility! !

!UIManager methodsFor: 'settings' stamp: 'Pavel.Krivanek 10/28/2008 10:49'!
interactiveParserFor: requestor	
	
	^ false! !

!UIManager methodsFor: 'ui process' stamp: 'MarcusDenker 12/2/2011 16:44'!
interruptName: labelString preemptedProcess: theInterruptedProcess
	"Create a Notifier on the active scheduling process with the given label."

	^ self error: 'Cannot perform a given request'! !

!UIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 6/17/2010 09:27'!
lowSpaceWatcherDefaultAction: preemptedProcess! !

!UIManager methodsFor: 'ui requests' stamp: 'StephaneDucasse 9/7/2011 21:21'!
multiLineRequest: queryString initialAnswer: defaultAnswer answerHeight: answerHeight
	"Create a multi-line instance of me whose question is queryString with
	the given initial answer. Invoke it centered at the given point, and
	answer the string the user accepts.  Answer nil if the user cancels.  An
	empty string returned means that the ussr cleared the editing area and
	then hit 'accept'.  Because multiple lines are invited, we ask that the user
	use the ENTER key, or (in morphic anyway) hit the 'accept' button, to 
	submit; that way, the return key can be typed to move to the next line."
	^self subclassResponsibility! !

!UIManager methodsFor: 'display' stamp: 'Pavel.Krivanek 10/28/2008 11:05'!
newDisplayDepthNoRestore: pixelSize	
	
	self subclassResponsibility ! !

!UIManager methodsFor: 'ui requests' stamp: 'StephaneDucasse 2/25/2011 18:44'!
newMenuIn: aThemedMorph for: aModel
	"Answer a new menu."
	
	^ self subclassResponsibility! !

!UIManager methodsFor: 'non-interactive' stamp: 'IgorStasenko 1/24/2011 16:31'!
nonInteractiveManager
	" Answer an instance of non-interactive manager, which will be used when image runs headless.
	  We put it here, so subclasses can override it. "
	^ NonInteractiveUIManager replacing: self! !

!UIManager methodsFor: 'events' stamp: 'Pavel.Krivanek 10/28/2008 11:01'!
onDebug: process context: context title: title full: bool! !

!UIManager methodsFor: 'events' stamp: 'Pavel.Krivanek 10/28/2008 11:22'!
onEventSensorStartup: anEventSensor

! !

!UIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 6/17/2010 09:24'!
onFontsChanged

	! !

!UIManager methodsFor: 'events' stamp: 'Pavel.Krivanek 10/28/2008 11:01'!
onPrimitiveError: aString	! !

!UIManager methodsFor: 'events' stamp: 'IgorStasenko 1/24/2011 15:56'!
onSnapshot: resuming

	"The resuming argument is true when image boots from disk,
	and false, if user just did an image snapshot."

	" do nothing by default "! !

!UIManager methodsFor: 'ui requests' stamp: 'FernandoOlivero 3/30/2011 15:47'!
openComparisionFrom: targetMethodSource
					to: originalMethodSource
					belongingTo: aClass
					from: aChange 
	^ self subclassResponsibility				! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:41'!
request: queryString 
	"Create an instance of me whose question is queryString. Invoke it 
	centered at the cursor, and answer the string the user accepts. Answer 
	the empty string if the user cancels."
	^self request: queryString initialAnswer: ''! !

!UIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 11/26/2010 10:18'!
request: queryString entryCompletion: anEntryCompletion
	"Create an instance of me whose question is queryString. Invoke it 
	centered at the cursor, and answer the string the user accepts. Answer 
	the empty string if the user cancels."
	^self request: queryString initialAnswer: '' entryCompletion: anEntryCompletion! !

!UIManager methodsFor: 'ui requests' stamp: 'StephaneDucasse 9/7/2011 21:51'!
request: queryString initialAnswer: defaultAnswer
	"Create an instance of me whose question is queryString with the given initial answer. Answer the string the user accepts. Answer the empty string if the user cancels."

	^self subclassResponsibility! !

!UIManager methodsFor: 'deprecated' stamp: 'StephaneDucasse 2/13/2010 11:22'!
request: queryString initialAnswer: defaultAnswer centerAt: aPoint
	"Create an instance of me whose question is queryString with the given
	initial answer. Invoke it centered at the given point, and answer the
	string the user accepts. Answer the empty string if the user cancels."

	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 11/26/2010 10:18'!
request: queryString initialAnswer: defaultAnswer entryCompletion: anEntryCompletion
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 10/19/2010 16:30'!
request: aStringOrText initialAnswer: defaultAnswer title: aTitle
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'AlainPlantec 11/25/2010 17:42'!
request: aStringOrText initialAnswer: defaultAnswer title: aTitle entryCompletion: anEntryCompletion
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."
	^self subclassResponsibility! !

!UIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:47'!
requestPassword: queryString
	"Create an instance of me whose question is queryString. Invoke it centered
	at the cursor, and answer the string the user accepts. Answer the empty 
	string if the user cancels."
	^self subclassResponsibility! !

!UIManager methodsFor: 'display' stamp: 'Pavel.Krivanek 10/28/2008 11:05'!
restoreDisplay	
	
	self subclassResponsibility ! !

!UIManager methodsFor: 'display' stamp: 'Pavel.Krivanek 10/28/2008 11:05'!
restoreDisplayAfter: aBlock	
	
	self subclassResponsibility ! !

!UIManager methodsFor: 'ui process' stamp: 'MarcusDenker 12/2/2011 16:35'!
spawnNewProcess.

	"do nothing by default"! !

!UIManager methodsFor: 'ui process' stamp: 'MarcusDenker 12/2/2011 16:35'!
spawnNewProcessIfThisIsUI: suspendedProcess

	self uiProcess == suspendedProcess ifTrue: [
		self spawnNewProcess.
		^true
	].
	^false		"no new process was created"
! !

!UIManager methodsFor: 'default actions' stamp: 'pavel.krivanek 6/17/2010 09:25'!
syntaxErrorNotificationDefaultAction: anException

	^ self subclassResponsibility ! !

!UIManager methodsFor: 'ui process' stamp: 'MarcusDenker 12/2/2011 16:30'!
terminateUIProcess
	"do nothing by default "! !

!UIManager methodsFor: 'private' stamp: 'FernandoOlivero 4/12/2011 09:56'!
theme  
	^ UITheme current! !

!UIManager methodsFor: 'default actions' stamp: 'IgorStasenko 1/25/2011 13:07'!
unhandledErrorDefaultAction: anException

	"Provide a default handling for unhandled error. 
	We should never reach this code, because it should be overridden by subclasses. 
	(but we can't put 'self subclassResponsibility' , because it will cause infinite loop,
	in case if current ui manager having no override of this method).
	This method is only for documentation purposes."
		
	Smalltalk snapshot: false andQuit: true.! !

!UIManager methodsFor: 'default actions' stamp: 'IgorStasenko 1/25/2011 13:07'!
warningDefaultAction: aWarning

	"Handle warning notification. Should be overidden by subclass"
	
	self subclassResponsibility  ! !
